#include <SnoopAdapter>
#include <SnoopTcpData>
#include <SnoopTcpBlock>
#include <iostream>
using namespace std;

int sendForwardBlock(pcap* pcap, SnoopPacket* packet, UINT8 flag, QByteArray msg = "")
{
  if (packet->proto != IPPROTO_TCP)
  {
    LOG_ERROR("packet->proto is %d", (int)packet->proto);
    return -1;
  }

  int bufSize = sizeof(ETH_HDR) + sizeof(IP_HDR) + sizeof(TCP_HDR) + msg.length();
  QByteArray ba; ba.resize(bufSize); char* buf = ba.data();

  ETH_HDR* ethHdr = (ETH_HDR*) &buf[0];
  IP_HDR*  ipHdr  = (IP_HDR*)  &buf[sizeof(ETH_HDR)];
  TCP_HDR* tcpHdr = (TCP_HDR*) &buf[sizeof(ETH_HDR) + sizeof(IP_HDR)];
  char*    data   = (char*)    &buf[sizeof(ETH_HDR) + sizeof(IP_HDR) + sizeof(TCP_HDR)];

  //
  // Ethernet Header
  //
  ethHdr->ether_dhost = packet->ethHdr->ether_shost;
  ethHdr->ether_shost = packet->ethHdr->ether_dhost;
  ethHdr->ether_type  = packet->ethHdr->ether_type;

  //
  // IP Header
  //
  ipHdr->ip_hl  = ???;
  ipHdr->ip_v   = ???;
  ipHdr->ip_tos = 0x44;
  ipHdr->ip_len = ???;
  ipHdr->ip_id  = ???;
  ipHdr->ip_off = ???;
  ipHdr->ip_ttl = ???;
  ipHdr->ip_p   = ???;
  ipHdr->ip_src = ???;
  ipHdr->ip_dst = ???;

  tcpHdr->th_sport = ???;
  tcpHdr->th_dport = ???;
  tcpHdr->th_seq   = ???;
  tcpHdr->th_ack   = ???;
  tcpHdr->th_off   = ???;
  tcpHdr->th_flags = ???;
  tcpHdr->th_x2    = ???;
  tcpHdr->th_win   = ???;
  tcpHdr->th_urp   = ???;

  //
  // Data
  //
  memcpy(data, msg.data(), msg.length());

  //
  // Checksum
  //
  tcpHdr->th_sum = htons(SnoopTcp::checksum(ipHdr, tcpHdr));
  ipHdr->ip_sum  = htons(SnoopIp::checksum(ipHdr));

  int res = pcap_sendpacket(pcap, (const u_char*)buf, bufSize);
  if (res < 0)
  {
    LOG_ERROR("pcap_sendpacket return %d", res);
  }
  return res;
}

int sendBackwardBlock(pcap* pcap, SnoopPacket* packet, UINT8 flag, QByteArray msg = "")
{
  return -1;
}

int main()
{
  SnoopAdapter adapter;
  adapter.autoRead = false;
  adapter.filter   = "dst port 80 && tcp[20:4]=0x47455420;

  if (!adapter.open())
  {
    cout << adapter.error.msg << endl;
    return -1;
  }

  while (true)
  {
    SnoopPacket packet;
    int readLen = adapter.read(&packet);
    if (readLen == 0) continue; // timeout
    if (readLen == -1) break;   // end of capturing

    if (!SnoopTcp::parseAll(&packet)) continue;
    if (packet.tcpHdr->th_flags & (TH_RST | TH_FIN)) continue;

    sendForwardBlock(adapter.m_pcap, &packet, TH_RST);
    //sendBackwardBlock(adapter.m_pcap, &packet, TH_FIN, "web site is blocked");
    LOG_DEBUG("TCP connection killed");
  }

  //return a.exec();
  return 0;
}
